package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.codegen.ExceptionLabel;

public abstract class SubRoutineStatement extends Statement
{
    ExceptionLabel anyExceptionLabel;
    
    public SubRoutineStatement() {
    }
    
    public static void reenterAllExceptionHandlers(SubRoutineStatement[] subroutines, int max, CodeStream codeStream) {
        if (subroutines == null) {
            return;
        }
        if (max < 0) {
            max = subroutines.length;
        }
        for (SubRoutineStatement sub : subroutines) {
            sub.enterAnyExceptionHandler(codeStream);
            sub.enterDeclaredExceptionHandlers(codeStream);
        }
        return;
    }
    
    public ExceptionLabel enterAnyExceptionHandler(CodeStream codeStream) {
        if (this.anyExceptionLabel == null) {
            this.anyExceptionLabel = new ExceptionLabel(codeStream, null);
        }
        this.anyExceptionLabel.placeStart();
        return this.anyExceptionLabel;
    }
    
    public void enterDeclaredExceptionHandlers(CodeStream codeStream) {
    }
    
    public void exitAnyExceptionHandler() {
        if (this.anyExceptionLabel != null) {
            this.anyExceptionLabel.placeEnd();
        }
    }
    
    public void exitDeclaredExceptionHandlers(CodeStream codeStream) {
    }
    
    public abstract boolean generateSubRoutineInvocation(BlockScope p0, CodeStream p1, Object p2, int p3, LocalVariableBinding p4);
    
    public abstract boolean isSubRoutineEscaping();
    
    public void placeAllAnyExceptionHandler() {
        this.anyExceptionLabel.place();
    }
}
